////////////////////////////////////////////
/////////////Hat registeration//////////////
////////////////////////////////////////////
package animatedHatsOverride {
	function HatMod_registerHat(%name, %dir, %offset, %eyeOffset, %animTime) {
		if(isHat(%name)) {
			echo("Error: Hat already exists! (" @ strReplace(%name, "_", " ") @ ")");
			HatMod_HatSet.add("Hat" @ %name @ "Data");
			return 0;
		}

		%evalString =	"datablock ShapeBaseImageData(Hat" @ %name @ "Data) {" SPC
							"shapeFile = \"" @ %dir @ "\";" SPC
							"mountPoint = $HeadSlot;" SPC
							"offset = \"" @ %offset @ "\";" SPC
							"eyeOffset = \"" @ %eyeOffset @ "\";" SPC
							"rotation = \"" @ eulerToMatrix("0 0 0") @ "\";" SPC
							"scale = \"0.1 0.1 0.1\";" SPC
							"doColorShift = false;" SPC
							"hatName = \"" @ %name @ "\";" SPC

							"stateName[0] = \"Activate\";" SPC
							"stateSequence[0] = loop;" SPC
							"stateTimeoutValue[0] = " @ %animTime @ ";" SPC
							"stateTransitionOnTimeout[0] = \"Activate2\";" SPC

							"stateName[1] = \"Activate2\";" SPC
							"stateSequence[1] = loop;" SPC
							"stateTimeoutValue[1] = " @ %animTime @ ";" SPC
							"stateTransitionOnTimeout[1] = \"Activate\";" SPC
						"};";

		eval(%evalString);
		HatMod_HatSet.add("Hat" @ %name @ "Data");

		if($Pref::HatMod::Items) {
			%evalString = 	"datablock ItemData(Hat" @ %name @ "ItemData) {" SPC
								"category = \"Hat\";" SPC
								"className = \"Hat\";" SPC
								"shapeFile = \"" @ %dir @ "\";" SPC
								"rotate = false;" SPC
								"mass = 1;" SPC
								"density = 0.2;" SPC
								"elasticity = 0.2;" SPC
								"friction = 0.6;" SPC
								"emap = true;" SPC
								"uiName = \"Hat" SPC strReplace(%name, "_", " ") @ "\";" SPC
								"iconName = \"Add-ons/Weapon_Bow/icon_bow\";" SPC
								"doColorShift = false;" SPC
								"image = Hat" @ %name @ "Data;" SPC
								"canDrop = true;" SPC
							"};";

			eval(%evalString); //We dont need to store these
		}

		return 1;
	}

	function HatMod_registerAllHats() {
		echo("Searching for Hats (Animated Override)...");

		if(isObject(HatMod_HatSet))
			HatMod_HatSet.clear();

		%loc = "Add-ons/HatMod_*/*/*.dts";
		for(%dir = findFirstFile(%loc); %dir !$= ""; %dir = findNextFile(%loc)) {
			//echo("DIR:" SPC %dir);
			%pos1 = strLastPos(%dir, "/")-1;
			%pos2 = strLastPos(getSubStr(%dir, 0, %pos1), "/");
			%name = getSubStr(%dir, 1+%pos2, %pos1-%pos2);
			//echo("NAME:" SPC %name);

			//echo("   Found hat:" SPC strReplace(%name, "_", " ") SPC "at dir" SPC %dir);

			%configDir = getSubStr(%dir, 0, strLastPos(%dir, "/")+1) @ %name @ ".txt";
			if(isFile(%configDir)) {
				echo("   Reading file for calibration.");

				%config = HatMod_GetProperties(%configDir);
			} else
				%config = HatMod_GetProperties("Default");

			%offset = getField(%config, 0);
			%minVer = getField(%config, 1);
			%eyeOffset = getField(%config, 2);
			%animTime = getField(%config, 3);

			if(%minVer > $HatMod::CurrentVersion) {
				echo("");
				warn("ERROR: Hat requires update to HatMod! (Required version" SPC %minVer @ ", current version" SPC $HatMod::CurrentVersion @ ")");
				echo("");
				continue;
			}

			if(HatMod_registerHat(%name, %dir, %offset, %eyeOffset, %animTime)) {
				echo("   Hat" SPC strReplace(%name, "_", " ") SPC "registered successfully.");
				%count++;
			}
		}

		%loc = "Add-ons/HatMod_*/*.dts";
		for(%dir = findFirstFile(%loc); %dir !$= ""; %dir = findNextFile(%loc)) {
			//echo("DIR:" SPC %dir);
			if(strPosCount(%dir, "/") != 2) //This filters out any that are Add-ons/HatMod_Whatever/Folder/File.dts
				continue;

			%name = getSubStr(%dir, 1+(%pos = strLastPos(%dir, "/")), strLen(%dir)-%pos-5);
			//echo("NAME:" SPC %name);

			//echo("   Found hat:" SPC strReplace(%name, "_", " ") SPC "at dir" SPC %dir);

			%configDir = getSubStr(%dir, 0, strLastPos(%dir, "/")+1) @ %name @ ".txt";
			if(isFile(%configDir)) {
				echo("   Reading file for calibration.");

				%config = HatMod_GetProperties(%configDir);
			} else
				%config = HatMod_GetProperties("Default");

			%offset = getField(%config, 0);
			%minVer = getField(%config, 1);
			%eyeOffset = getField(%config, 2);
			%animTime = getField(%config, 3);

			if(%minVer > $HatMod::CurrentVersion) {
				echo("");
				warn("ERROR: Hat requires update to HatMod! (Required version" SPC %minVer @ ", current version" SPC $HatMod::CurrentVersion @ ")");
				echo("");
				continue;
			}

			if(HatMod_registerHat(%name, %dir, %offset, %eyeOffset, %animTime)) {
				echo("   Hat" SPC strReplace(%name, "_", " ") SPC "registered successfully.");
				%count++;
			}
		}

		%loc = "Add-ons/SimpleWell_*/ITEMS/SHAPES/*.dts";
		for(%dir = findFirstFile(%loc); %dir !$= ""; %dir = findNextFile(%loc)) {
			//echo("DIR:" SPC %dir);
			%name = getSubStr(%dir, 1+(%pos = strLastPos(%dir, "/")), strLen(%dir)-%pos-5);
			//echo("NAME:" SPC %name);

			//echo("   Found SW hat:" SPC strReplace(%name, "_", " ") SPC "at dir" SPC %dir);

			%configDir = getSubStr(%dir, 0, strLastPos(%dir, "/")-6) @ %name @ ".cs";

			if(isFile(%configDir)) {
				echo("   Reading file for calibration.");

				%config = HatMod_GetProperties(%configDir);
			} else
				%config = HatMod_GetProperties("Default");

			%offset = getField(%config, 0);
			%minVer = getField(%config, 1);
			%eyeOffset = getField(%config, 2);
			%animTime = getField(%config, 3);

			if(%minVer > $HatMod::CurrentVersion) {
				echo("");
				warn("ERROR: Hat requires update to HatMod! (Required version" SPC %minVer @ ", current version" SPC $HatMod::CurrentVersion @ ")");
				echo("");
				continue;
			}

			if(HatMod_registerHat(%name, %dir, %offset, %eyeOffset, %animTime)) {
				echo("   Hat" SPC strReplace(%name, "_", " ") SPC "registered successfully.");
				%count++;
			}
		}

		echo("Hat search done. Found" SPC %count+0 SPC "new hats.");
	}

	function HatMod_GetProperties(%configDir) {
		%offset = "0 0 0";
		%minVer = "0";
		%eyeOffset = "0 0 -1000";
		%animTime = 1;

		if(%configDir !$= "Default") {
			%file = new fileObject();
			%file.openForRead(%configDir);

			while(!%file.isEOF()) {
				%line = trim(%file.readLine());

				if(%line $= "")
					continue;

				%firstWord = getWord(%line, 0);
				%wordCount = getWordCount(%line);

				switch$(%firstWord) {
				case "offset":
					%offset = getWords(%line, %wordCount-3, %wordCount-1);
					%offset = strReplace(%offset, "\"", "");
					%offset = strReplace(%offset, ";", "");
					%offset = vectorAdd(%offset, "0 0 0"); //Forces it to be a 3d vector

				case "minVer":
					%minVer = getWord(%line, getWordCount(%line)-1);

				case "eyeOffset":
					%eyeOffset = getWords(%line, %wordCount-3, %wordCount-1);
					%eyeOffset = strReplace(%eyeOffset, "\"", "");
					%eyeOffset = strReplace(%eyeOffset, ";", "");
					%eyeOffset = vectorAdd(%eyeOffset, "0 0 0"); //Forces it to be a 3d vector

				case "animTime":
					%animTime = getWord(%line, %wordCount-1) / 1000;
				}
			}

			%file.close();
			%file.delete();
		}

		return %offset TAB %minVer TAB %eyeOffset TAB %animTime;
	}

};
activatePackage(animatedHatsOverride);